@@ -1,7 +1,6 @@
-from pathlib import Path
-from metaDMG.utils import load_config, get_results_dir
 import pandas as pd
-from metaDMG.fit.results import append_fit_predictions
+
+from metaDMG.utils import append_fit_predictions, get_results_dir
 
 
 def load_results(config_path=None, results_dir=None):
@@ -13,8 +12,10 @@ def load_results(config_path=None, results_dir=None):
     return df_results
 
 
-def filter_results(df_results, query):
+def filter_results(df_results: pd.DataFrame, query: str):
     if query:
+        if query.startswith(" & "):
+            query = query[3:]
         df_results = df_results.query(query)
     return df_results
 
