@@ -1,13 +1,13 @@
 import math
+from pathlib import Path
+
 import numpy as np
 import pandas as pd
+import yaml
 from iminuit import describe
 from numba import njit
 from scipy.special import erf, erfinv
-from scipy.stats import beta as sp_beta
 from scipy.stats import chi2 as sp_chi2
-import yaml
-from pathlib import Path
 
 
 #%%
@@ -21,41 +21,175 @@ for ref in ACTG:
 
 #%%
 
+from itertools import islice
+from typing import Iterable, Iterator, Optional
 
-def get_samples_from_config(config):
-    return list(config["samples"].keys())
+import psutil
+import typer
+from logger_tt import logger
 
 
-def get_config_from_sample(sample, config, forced=False):
-    config = config.copy()
-    config["sample"] = sample
-    config["bam"] = config["samples"][sample]
+class Config(dict):
+    """Config contains the parameters related to specific alignment file."""
 
-    dir_lca = Path(config["dir"]) / "lca"
-
-    config["path_mismatches_txt"] = str(dir_lca / f"{sample}.mismatches.txt.gz")
-    if config["damage_mode"] == "lca":
-        config["path_mismatches_stat"] = str(
-            dir_lca / f"{sample}.mismatches.stat.txt.gz"
-        )
-    else:
-        config["path_mismatches_stat"] = str(dir_lca / f"{sample}.stat.txt")
-    config["path_lca"] = str(dir_lca / f"{sample}.lca.txt.gz")
-    config["path_lca_log"] = str(dir_lca / f"{sample}.log.txt")
-
-    dir_tmp = Path(config["dir"]) / "tmp" / sample
-    config["path_tmp"] = dir_tmp
-
-    return config
+    pass
 
 
-def make_configs(config):
-    samples = get_samples_from_config(config)
-    configs = [get_config_from_sample(sample, config) for sample in samples]
-    return configs
+class Configs(dict):
+    """Configs contains the parameters related to config file.
+    Inherits from dict. Implements iterations.
+    """
+
+    def __iter__(self) -> Iterator[Config]:
+        """Iteration
+
+        Yields
+        ------
+        Iterator[Config]
+            Allow for iteration
+        """
+        dir_lca = self["dir"] / "lca"
+        samples = self["samples"].keys()
+        for sample in samples:
+            config = Config(self)
+            config["sample"] = sample
+            config["bam"] = config["samples"][sample]
+
+            config["path_mismatches_txt"] = dir_lca / f"{sample}.mismatches.txt.gz"
+
+            if config["damage_mode"] == "lca":
+                config["path_mismatches_stat"] = (
+                    dir_lca / f"{sample}.mismatches.stat.txt.gz"
+                )
+            else:
+                config["path_mismatches_stat"] = dir_lca / f"{sample}.stat.txt"
+
+            config["path_lca"] = dir_lca / f"{sample}.lca.txt.gz"
+            config["path_lca_log"] = dir_lca / f"{sample}.log.txt"
+            config["path_tmp"] = config["dir"] / "tmp" / sample
+            yield config
+
+    def get_nth(self, n: int) -> Config:
+        """Gets the n'th config
+
+        Parameters
+        ----------
+        n
+            The index
+
+        Returns
+        -------
+        Config
+            A single configuration
+        """
+        return next(islice(self, n, None))
+
+    def get_first(self) -> Config:
+        """Get the first config
+
+        Returns
+        -------
+        Config
+            A single configuration
+        """
+        return self.get_nth(n=0)
+
+    def __len__(self) -> int:
+        """The number of configs
+
+        Returns
+        -------
+        int
+            The number of configs
+        """
+        return len(self["samples"].keys())
+
+    def check_number_of_jobs(self) -> None:
+        """Compare the number of configs to the number of cores used."""
+
+        cores = min(self["cores"], len(self["samples"]))
+        cores_pr_fit = self["cores_pr_fit"]
+        N_jobs = cores * cores_pr_fit
+        max_cores = psutil.cpu_count(logical=True)
+        max_cores_real = psutil.cpu_count(logical=False)
+
+        if N_jobs > max_cores:
+            logger.warning(
+                f"The total number of jobs {N_jobs} are higher "
+                f"than the number of cores {max_cores}. "
+                f"Do not do this unless you know what you are doing. "
+                f"Try decreasing either 'cores' or 'cores-pr-fit'."
+            )
+        elif N_jobs > max_cores_real:
+            logger.info(
+                f"The total number of jobs {N_jobs} are higher "
+                f"than the real number of cores {max_cores_real} (non-logical). "
+                f"This might decrease performance. "
+            )
 
 
-#%%
+def make_configs(
+    config_path: Optional[Path],
+    log_port: Optional[int] = None,
+    log_path: Optional[str] = None,
+    forced: bool = False,
+) -> Configs:
+    """Create an instance of Configs from a config file
+
+    Parameters
+    ----------
+    config_path
+        The config file to load
+    log_port
+        Optional log port, by default None
+    log_path
+        Optional log path, by default None
+    forced
+        Whether or not the computations are forced, by default False
+
+    Returns
+    -------
+        An instance of Configs
+
+    Raises
+    ------
+    typer.Abort
+        If not a proper config file
+    """
+
+    if config_path is None:
+        config_path = Path("config.yaml")
+
+    if not config_path.exists():
+        logger.error("Error! Please select a proper config file!")
+        raise typer.Abort()
+
+    logger.info(f"Using {config_path} as config file.")
+    with open(config_path, "r") as file:
+        d = yaml.safe_load(file)
+
+    d["log_port"] = log_port
+    d["log_path"] = log_path
+    if forced:
+        d["forced"] = True
+
+    d.setdefault("forward_only", False)
+    d.setdefault("cores_pr_fit", 1)
+    d.setdefault("damage_mode", "lca")
+    d.setdefault("forced", False)
+
+    paths = ["names", "nodes", "acc2tax", "dir", "config_path"]
+    for path in paths:
+        d[path] = Path(d[path])
+    for key, val in d["samples"].items():
+        d["samples"][key] = Path(val)
+
+    for key, val in d.items():
+        if isinstance(val, str):
+            if val.isdigit():
+                d[key] = int(key)
+
+    return Configs(d)
 
 
 #%%
@@ -100,10 +234,12 @@ def downcast_dataframe(df, categories=None, fully_automatic=False):
 def is_forward(df):
     return df["direction"] == "5'"
 
+
 def get_forward(df):
     s = "5'"
     return df.query("direction == @s")
 
+
 def get_priors():
 
     # beta
@@ -182,7 +318,7 @@ def gammaln_vec(xs):
 @njit
 def log_betabinom_PMD(k, N, alpha, beta):
     return (
-        gammaln_vec(N + 1)
+        gammaln_vec(N + 1)  # type: ignore
         + gammaln_vec(k + alpha)
         + gammaln_vec(N - k + beta)
         + gammaln_vec(alpha + beta)
