@@ -1,14 +1,17 @@
 #%%
-from collections import defaultdict
+import itertools
 import warnings
+from collections import defaultdict
+from multiprocessing import Pool
+
 import joblib
 import numpy as np
 import numpyro
 import pandas as pd
-from multiprocessing import Pool
-import itertools
 from logger_tt import logger
-from metaDMG.fit import bayesian, frequentist, fit_utils
+
+from metaDMG.fit import bayesian, fit_utils, frequentist
+
 
 numpyro.enable_x64()
 
@@ -389,7 +392,7 @@ def read_stats_lca(config):
     data = data.replace("\t\t", "\t'")
     data = data.replace('1:root:"no rank"', """1:root:"no rank"'""")
 
-    string_columns = ["tax_name", "tax_rank", "tax_path"]
+    string_columns = ["tax_id", "tax_name", "tax_rank", "tax_path"]
 
     df_stats = pd.read_csv(
         StringIO(data),  # config["path_mismatches_stat"]
