@@ -1,17 +1,23 @@
 #%%
-import pandas as pd
-from pathlib import Path
-import subprocess
 import shlex
 import shutil
-from logger_tt import logger
-from multiprocessing import current_process
+import subprocess
 
 # import json
 from collections import Counter
+from multiprocessing import current_process
+from pathlib import Path
+from typing import Optional, Tuple
+
+import pandas as pd
+from logger_tt import logger
+
+from metaDMG import utils
+from metaDMG.errors import Error, MismatchFileError, metadamageError
+from metaDMG.fit import fits, mismatches, results
+from metaDMG.fit.fit_utils import Config
 from metaDMG.loggers.loggers import setup_logger
-from metaDMG.errors import metadamageError, Error, MismatchFileError
-from metaDMG.fit import mismatches, fits, results
+
 
 #%%
 
@@ -37,15 +43,15 @@ def do_load(targets, forced=False):
     return not do_run(targets, forced=forced)
 
 
-def data_dir(config, name, suffix="parquet"):
-    target = Path(config["dir"]) / name / f"{config['sample']}.{name}.{suffix}"
-    return str(target)
+def data_dir(config: Config, name, suffix="parquet"):
+    target = config["dir"] / name / f"{config['sample']}.{name}.{suffix}"
+    return target
 
 
 #%%
 
 
-def get_LCA_command(config):
+def get_LCA_command(config: Config) -> str:
     outnames = config["path_tmp"] / config["sample"]
     lca_rank = f"-lca_rank {config['lca_rank']}" if config["lca_rank"] != "" else ""
 
@@ -70,7 +76,7 @@ def get_LCA_command(config):
     return command[:-1]
 
 
-def get_LCA_mismatches_command(config):
+def get_LCA_mismatches_command(config: Config) -> str:
     sample = config["sample"]
     bdamage = config["path_tmp"] / f"{sample}.bdamage.gz"
     lca_stat = config["path_tmp"] / f"{sample}.stat"
@@ -88,7 +94,7 @@ def get_LCA_mismatches_command(config):
 #%%
 
 
-def get_runmode(config):
+def get_runmode(config: Config) -> int:
 
     runmode = config["damage_mode"]
 
@@ -106,7 +112,7 @@ def get_runmode(config):
     raise AssertionError(f"Got wrong runmode. Got: {runmode}")
 
 
-def get_damage_command(config):
+def get_damage_command(config: Config) -> str:
     "'Direct' damage, no LCA"
 
     outname = config["path_tmp"] / config["sample"]
@@ -124,16 +130,16 @@ def get_damage_command(config):
     return command[:-1]
 
 
-def get_damage_ugly_command(config):
+def get_damage_ugly_command(config: Config) -> str:
     bdamage = config["path_tmp"] / f"{config['sample']}.bdamage.gz"
-    command = f"{config['metaDMG-lca']} print_ugly " f"{bdamage} "
-    return command[:-1]
+    command = f"{config['metaDMG-lca']} print_ugly {bdamage} -bam {config['bam']}"
+    return command
 
 
 #%%
 
 
-def move_files(config):
+def move_files(config: Config) -> None:
 
     sample = config["sample"]
     path_tmp = config["path_tmp"]
@@ -149,13 +155,13 @@ def move_files(config):
     }
     for source_path, target_path in d_move_source_target.items():
         logger.debug(f"Moving {source_path} to {target_path}.")
-        if not Path(source_path).is_file():
+        if not source_path.is_file():
             raise metadamageError(f"{source_path} does not exist.")
-        Path(target_path).parent.mkdir(parents=True, exist_ok=True)
+        target_path.parent.mkdir(parents=True, exist_ok=True)
         shutil.move(source_path, target_path)
 
 
-def move_files_non_lca(config):
+def move_files_non_lca(config: Config) -> None:
 
     sample = config["sample"]
     path_tmp = config["path_tmp"]
@@ -169,37 +175,34 @@ def move_files_non_lca(config):
     }
     for source_path, target_path in d_move_source_target.items():
         logger.debug(f"Moving {source_path} to {target_path}.")
-        if not Path(source_path).is_file():
+        if not source_path.is_file():
             raise metadamageError(f"{source_path} does not exist.")
-        Path(target_path).parent.mkdir(parents=True, exist_ok=True)
+        target_path.parent.mkdir(parents=True, exist_ok=True)
         shutil.move(source_path, target_path)
 
 
 #%%
 
 
-def create_tmp_dir(config):
-    Path(config["path_tmp"]).mkdir(parents=True, exist_ok=True)
+def create_tmp_dir(config: Config) -> None:
+    config["path_tmp"].mkdir(parents=True, exist_ok=True)
 
 
-def delete_tmp_dir(config):
-    path_tmp = Path(config["path_tmp"])
-    for file in path_tmp.glob("*"):
-        file.unlink()
-    path_tmp.rmdir()
+def delete_tmp_dir(config: Config) -> None:
+    utils.remove_directory(config["path_tmp"])
 
 
 #%%
 
 
-def run_command(command):
+def run_command(command: str):
 
     p = subprocess.Popen(
         shlex.split(command),
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
     )
-    for line in iter(p.stdout.readline, b""):
+    for line in iter(p.stdout.readline, b""):  # type: ignore
         if line:
             line = line.decode("utf-8")
             if line.endswith("\n"):
@@ -226,7 +229,7 @@ def handle_returncode(command, line, counter):
     return None
 
 
-def run_command_helper(config, command):
+def run_command_helper(config: Config, command: str) -> None:
 
     # add a counter to avoid too many similar lines
     counter = Counter()
@@ -253,7 +256,7 @@ def run_command_helper(config, command):
 #%%
 
 
-def run_LCA(config, forced=False):
+def run_LCA(config: Config, forced: bool = False) -> None:
 
     logger.info(f"Getting LCA.")
 
@@ -287,7 +290,7 @@ def run_LCA(config, forced=False):
 #%%
 
 
-def run_damage_no_lca(config, forced=False):
+def run_damage_no_lca(config: Config, forced: bool = False) -> None:
 
     logger.info(f"Getting damage.")
 
@@ -320,7 +323,7 @@ def run_damage_no_lca(config, forced=False):
 #%%
 
 
-def run_thorfinn(config, forced=False):
+def run_cpp(config: Config, forced: bool = False) -> None:
     if config["damage_mode"] == "lca":
         run_LCA(config, forced=forced)
     else:
@@ -330,7 +333,7 @@ def run_thorfinn(config, forced=False):
 #%%
 
 
-def get_df_mismatches(config, forced=False):
+def get_df_mismatches(config: Config, forced: bool = False) -> pd.DataFrame:
 
     logger.info(f"Getting df_mismatches")
 
@@ -339,7 +342,7 @@ def get_df_mismatches(config, forced=False):
     if do_run(target, forced=forced):
         logger.info(f"Computing df_mismatches.")
         df_mismatches = mismatches.compute(config)
-        Path(target).parent.mkdir(parents=True, exist_ok=True)
+        target.parent.mkdir(parents=True, exist_ok=True)
         df_mismatches.to_parquet(target)
 
     else:
@@ -352,11 +355,15 @@ def get_df_mismatches(config, forced=False):
 #%%
 
 
-def dataframe_columns_contains(df, s):
+def dataframe_columns_contains(df: pd.DataFrame, s: str) -> bool:
     return any(s in column for column in df.columns)
 
 
-def get_df_fit_results(config, df_mismatches, forced=False):
+def get_df_fit_results(
+    config: Config,
+    df_mismatches: pd.DataFrame,
+    forced: bool = False,
+) -> pd.DataFrame:
 
     logger.info(f"Getting df_fit_results.")
 
@@ -384,7 +391,7 @@ def get_df_fit_results(config, df_mismatches, forced=False):
 
     logger.info(info)
     df_fit_results = fits.compute(config, df_mismatches)
-    Path(target).parent.mkdir(parents=True, exist_ok=True)
+    target.parent.mkdir(parents=True, exist_ok=True)
     df_fit_results.to_parquet(target)
 
     return df_fit_results
@@ -393,7 +400,12 @@ def get_df_fit_results(config, df_mismatches, forced=False):
 #%%
 
 
-def get_df_results(config, df_mismatches, df_fit_results, forced=False):
+def get_df_results(
+    config: Config,
+    df_mismatches: pd.DataFrame,
+    df_fit_results: pd.DataFrame,
+    forced: bool = False,
+) -> pd.DataFrame:
 
     logger.info(f"Getting df_results.")
 
@@ -414,7 +426,7 @@ def get_df_results(config, df_mismatches, df_fit_results, forced=False):
     # Compute the results:
     logger.info(f"Computing df_results.")
     df_results = results.merge(config, df_mismatches, df_fit_results)
-    Path(target).parent.mkdir(parents=True, exist_ok=True)
+    target.parent.mkdir(parents=True, exist_ok=True)
     df_results.to_parquet(target)
 
     return df_results
@@ -423,8 +435,7 @@ def get_df_results(config, df_mismatches, df_fit_results, forced=False):
 #%%
 
 
-def run_single_config(config):
-
+def _setup_logger(config: Config) -> None:
     # if not main process (and haven't been initialized before)
     name = current_process().name
     if "SpawnPoolWorker" in name or "SpawnProcess" in name:
@@ -432,17 +443,38 @@ def run_single_config(config):
             log_port=config["log_port"],
             log_path=config["log_path"],
         )
-
     current_process().name = config["sample"]
 
-    if not Path(config["bam"]).is_file():
+
+def BAM_file_is_valid(config: Config) -> bool:
+
+    if not config["bam"].is_file():
         logger.error(f"The sample bam file does not exist: {config['bam']}.")
+        return False
+
+    if config["bam"].stat().st_size == 0:
+        logger.error(f"The sample bam file is of size 0: {config['bam']}.")
+        return False
+
+    return True
+
+
+#%%
+
+
+def run_single_config(
+    config: Config,
+) -> Optional[Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]]:
+
+    _setup_logger(config)
+
+    if not BAM_file_is_valid(config):
         return None
 
     forced = config["forced"]
 
     try:
-        run_thorfinn(config, forced=forced)
+        run_cpp(config, forced=forced)
     except metadamageError:
         logger.exception(
             f"{config['sample']} | metadamageError with run_LCA. "
