@@ -1,18 +1,15 @@
 # from multiprocessing import Pool
 from concurrent.futures import ProcessPoolExecutor as Pool
-from logger_tt import logger
 from datetime import datetime
-from metaDMG.fit import serial, fit_utils
-from metaDMG.utils import check_number_of_jobs
+
+from logger_tt import logger
+
+from metaDMG.fit import fit_utils, serial
 
 
-def run_workflow(config):
+def run_workflow(configs: fit_utils.Configs):
 
-    configs = fit_utils.make_configs(config)
-
-    cores = config["cores"]
-    if cores > len(configs):
-        cores = len(configs)
+    cores = min(configs["cores"], len(configs))
 
     logger.info(f"Running metaDMG on {len(configs)} files in total.")
 
@@ -24,7 +21,7 @@ def run_workflow(config):
 
     else:
         logger.info(f"Running with {cores} processes in parallel")
-        check_number_of_jobs(config)
+        configs.check_number_of_jobs()
 
         with Pool(max_workers=cores) as pool:
             # for dfs in pool.imap_unordered(serial.run_single_config, configs):
